(* ------------------- PBMIN.INC -----------------------

  Include file to load a screen dump from a PBM (portable bitmap) file.
  Supported resolutions: 720x256, 720x512
  Provides the following functions:

    procedure LoadScreenFromPBM -> Get image row from a PBM file
                                   and copy it to the Roller RAM

  Uses include file KERNEL.INC.

  ---------------------------------------------------- *)
const
  BufSize = 128;

type
  TFileName  = string(.14.);
  BinaryFile = file;
  ErrorCode  = (ErrOk, ErrFileNotFound, ErrFormatNotSupported);

var
  fbuf:         Array(.1..BufSize.) of byte;
  fbufidx:      byte;
  recsread:     integer;
  pbmfile:      BinaryFile;
  headerOk:     boolean;
  doubleHeight: boolean;
  result:       ErrorCode;


(* Reads a byte from the buffer. Reads a block from disc
   as soon as the buffer is read to the end. *)
function ReadBuf: byte;
begin
  if fbufidx = BufSize then begin
    fbufidx := 1;
    BlockRead(pbmfile, fbuf, 1, recsread); (* reads a block of 128 bytes *)
  end
  else
    fbufidx := succ(fbufidx);
  ReadBuf := fbuf(.fbufidx.);
end; (* ReadBuf *)

(* Checks whether the next value from the buffer equals
   the byte in the parameter. *)
function EqualsNextBufVal(b: byte): boolean;
begin
    EqualsNextBufVal := (ReadBuf = b);
end; (* EqualsNextBufVal *)


(* Reads a line of pixels *)
procedure ReadPixelLine(y: byte);
var
  x: byte;
  i: integer;
begin
  i := y;
  for x := 1 to 90 do begin (* 90 = 720 / 8 *)
    GX_Buffer(.i.) := ReadBuf;
    i := i + 8;
  end;
end; (* ReadPixelLine *)


(* Loads a screen dump from a black and white 1 bit per pixel PBM file. *)
function LoadScreenFromPBM(filename: TFileName): ErrorCode;
var
  row:  byte;
  x, y: byte;
  i:    integer;
  b:    byte;
begin
  fbufidx := BufSize;
  headerOk := false;
  doubleHeight := false;

  Assign(pbmfile, filename);
  (*$I-*)
  Reset(pbmfile);
  (*$I+*)
  if IOResult <> 0 then begin
    result := ErrFileNotFound;
  end else begin
    (* Read header *)
    if EqualsNextBufVal($50) and (* 'P' *)
       EqualsNextBufVal($34) and (* '4' *)
       EqualsNextBufVal($0A) and (* LF  *)
       EqualsNextBufVal($37) and (* '7' *)
       EqualsNextBufVal($32) and (* '2' *)
       EqualsNextBufVal($30)     (* '0' *)
    then begin
       b := ReadBuf;
       if (b = $20) or (b = $0A) then begin      (* ' ' or LF *)
         b := ReadBuf;
         if b = $32 then begin (* 720x256 *)     (* '2' *)
           doubleHeight := false;
           if EqualsNextBufVal($35) and          (* '5' *)
              EqualsNextBufVal($36) then begin   (* '6' *)
              headerOk := EqualsNextBufVal($0A); (* LF  *)
           end;
         end else
         if b = $35 then begin (* 720x512 *)     (* '5' *)
           doubleHeight := true;
           if EqualsNextBufVal($31) and          (* '1' *)
              EqualsNextBufVal($32) then begin   (* '2' *)
              headerOk := EqualsNextBufVal($0A); (* LF  *)
           end;
         end;
       end;
    end;

    if headerOk then begin
      (* Read pixel data *)
      for row := 0 to 31 do begin
        for y := 0 to 7 do begin
          ReadPixelLine(y);
          if doubleHeight then ReadPixelLine(y);
        end;
        PutScreenRow(row); (* puts 720 bytes = 8 rows of 90 bytes *)
      end;
      result := ErrOk;
    end else begin
      result := ErrFormatNotSupported;
    end;
    
    Close(pbmfile);
  end;
  
  LoadScreenFromPBM := Result;
end; (* LoadScreenFromPBM *)
